# -*- coding: utf-8 -*-
"""qt.gui.py"""

# Copyright (C) 2009-2017 Federico Brega, Pierluigi Villani

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

from PyQt5 import QtCore
from PyQt5.QtGui import *
from PyQt5.QtWidgets import *
from . import ui_qt
import glal
import os.path
from themes import ThemeManager
from version import VERSION, VERSION_CHECK_ENABLED
import io
from functools import reduce

class Gui(QMainWindow, ui_qt.Ui_MainWindow):
    """Main frame"""
    def __init__(self, parent, title, size):
        QMainWindow.__init__(self, parent)
        self.setupUi(self)
        self.setWindowTitle(title)
        self.notebook.removeTab(0)
        self.cg_exit = None  # check on exit callback
        self.setContextMenuPolicy(QtCore.Qt.PreventContextMenu)
        self.resize(QtCore.QSize(size[0], size[1]))

        # center Gui
        screen = QDesktopWidget().screenGeometry()
        dimension = self.geometry()
        self.move((screen.width() - dimension.width()) / 2,
                  (screen.height() - dimension.height()) / 2)
        if size[2]:
            self.showMaximized()

        # Set standard icons
        icon1 = QIcon(QPixmap(":/qt-icons/pixmaps/open.png"))
        icon1 = QIcon.fromTheme("document-open",  icon1)
        self.menu_item_open.setIcon(icon1)
        icon2 = QIcon(QPixmap(":/qt-icons/pixmaps/save_as.png"))
        icon2 = QIcon.fromTheme("document-save-as",  icon2)
        self.menu_item_save_as.setIcon(icon2)
        icon0 = QIcon(QPixmap(":/qt-icons/pixmaps/save.png"))
        icon0 = QIcon.fromTheme("document-save",  icon0)
        self.menu_item_save.setIcon(icon0)
        icon3 = QIcon(QPixmap(":/qt-icons/pixmaps/new.png"))
        icon3 = QIcon.fromTheme("document-new",  icon3)
        self.menu_item_new.setIcon(icon3)
        icon_about = QIcon.fromTheme("help-about")
        self.menu_item_about.setIcon(icon_about)
        icon_bug = QIcon.fromTheme("tools-report-bug")
        self.menu_item_bug.setIcon(icon_bug)
        icon_exit = QIcon.fromTheme("application-exit")
        self.menu_item_exit.setIcon(icon_exit)
        icon_server = QIcon.fromTheme("network-server")
        self.menu_Kml_server.setIcon(icon_server)
        icon_pref = QIcon.fromTheme("preferences-desktop")
        self.preferences.setIcon(icon_pref)

        # Group the altitude services to have radio button.
        serv_Action_group = QActionGroup(self)
        serv_Action_group.addAction(self.menu_item_s1)
        serv_Action_group.addAction(self.menu_item_s2)
        serv_Action_group.addAction(self.menu_item_s3)

        self.menu_item_about.triggered.connect(self.onabout)

    def closeEvent(self, event):
        """If a close event is generated call the controller"""
        if self.cg_exit():
            event.accept()
        else:
            event.ignore()

    def quit(self):
        """ Exit from CycloGraph"""
        self.close()

    def getdimensions(self):
        """Return window width and height"""
        return (self.size().width(), self.size().height())

    def ismaximized(self):
        """Return true if window is maximized"""
        return self.isMaximized()

    def addstatusbartext(self, text):
        """Show text in the statusbar for 5000 ms"""
        self.statusbar.showMessage(text, 5000)

    def onabout(self):
        """About dialog"""
        description = """CycloGraph"""
        licence = """
CycloGraph is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

CycloGraph is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
You should have received a copy of the GNU General Public License along with this program. If not, see http://www.gnu.org/licenses/.
"""
        text = 'Copyright (C) 2008-2017 Federico Brega, Pierluigi Villani\n' + \
        'http://cyclograph.sourceforge.net\n' + licence
        QMessageBox.about(self, description +" "+ VERSION, text)

class Page(QFrame, ui_qt.Ui_Frame_Page):
    """Page showed in a tab"""
    def __init__(self, parent):
        QFrame.__init__(self, parent)
        self.savedfilepath = None
        self.slopecounternew = 0
        self.modified = False
        self.setupUi(self)

    def insert_row(self, row_num, cp):
        """Insert item to this page at position row_num"""
        self.tableWidget.insertRow(row_num)
        for col in range(len(cp)):
            item = QTableWidgetItem(str(cp[col]))
            self.tableWidget.setItem(row_num, col, item)

    def delete_row(self, row_num):
        """Removes row at position row_num in this page."""
        self.tableWidget.removeRow(row_num)

    def get_sel_row(self):
        """Gives the row number that is selected."""
        row = self.tableWidget.currentRow()
        return row

    def get_sel_multi_row(self):
        """Gives a list with the row number of selected items."""
        rows = [self.tableWidget.row(item) \
                for item in self.tableWidget.selectedItems()]
        rows = list(set(rows))
        rows.sort()
        return rows

class Managecp(QDialog, ui_qt.Ui_manag_dlg):
    """Add and edit dialog"""
    def __init__(self, parent, title,
                 dist="", alt="", name=""):
        QDialog.__init__(self, parent)
        self.setupUi(self)
        self.setWindowTitle(title)
        self.distance.setText(dist)
        self.altitude.setText(alt)
        self.name.setText(name)

    def show_modal(self):
        """Shows the dialog and returns True if the value has to be chenged."""
        val = self.exec_()
        return val == QDialog.Accepted

    def destroy(self):
        """Destroy current managecp"""
        pass

    def getcp(self):
        """Gives the modified values."""
        dist = self.distance.text()
        alt = self.altitude.text()
        name = self.name.text()
        return (dist, alt, name)

class FormSlopeInfo(QDialog, ui_qt.Ui_Slope_Dialog):
    """info dialog about the slope"""
    def __init__(self, parent, slopedata):
        QDialog.__init__(self, parent)
        self.setupUi(self)
        self.s_name.setText(slopedata['name'])
        self.s_state.setText(slopedata['state'])
        self.s_author.setText(slopedata['author'])
        self.s_email.setText(slopedata['email'])
        self.average_grad.setText(slopedata['average_grad'])
        self.max_grad.setText(slopedata['max_grad'])
        self.height_difference.setText(slopedata['height_difference'])
        self.height_gain.setText(slopedata['height_gain'])
        self.s_comment.setText(slopedata['comment'])

    def getslopeinfo(self):
        """Returns the slope infos: name, state, comment"""
        return {'name': self.s_name.text(),
                'state': self.s_state.text(),
                'author': self.s_author.text(),
                'email': self.s_email.text(),
                'comment': self.s_comment.toPlainText()}

FONT_TYP = {
            QFont.Light: "light",
            QFont.Normal: "normal",
            QFont.Bold: "bold",
            QFont.DemiBold: "bold",
            QFont.Black: "bold",
           }
class Preferences(QDialog, ui_qt.Ui_Pref_Dialog):

    """Set gradient's color"""
    def __init__(self, parent, pref, default_pref):
        QDialog.__init__(self, parent)
        self.setupUi(self)

        self.default = default_pref
        # General
        self.olines.setChecked(pref['olines'])
        self.effect3d.setChecked(pref['3d'])
        self.legend.setChecked(pref['legend'])
        self.showinfo.setChecked(pref['sinfo'])
        self.vcheck.setChecked(pref['vcheck'])
        if not VERSION_CHECK_ENABLED:
            self.label.hide()
            self.vcheck.hide()
        res = 0
        if pref['width'] == 1600:
            res = 3
        if pref['width'] == 1280:
            res = 2
        elif pref['width'] == 1024:
            res = 1
        self.res.setCurrentIndex(res)

        current = 0
        for theme in pref['themelist']:
            self.theme.addItem("")
            self.theme.setItemText(current, theme)
            if theme == pref['theme']:
                self.theme.setCurrentIndex(current)
            current += 1

        self.serv.setCurrentIndex(0)
        if pref['serv'] == 'usgs.net':
            self.serv.setCurrentIndex(2)
        elif pref['serv'] == 'earthtools.org':
            self.serv.setCurrentIndex(1)
        # Fonts
        self.fontbuttonlist = [self.fdesc,
                               self.fgrad,
                               self.ftitle]
        self.fontbuttongroup = QButtonGroup(parent)
        for id_, button in enumerate(self.fontbuttonlist):
            self.fontbuttongroup.addButton(button, id_)
        desc = pref['fdesc']
        self.fdesc.setText(desc['des']+' | '+str(desc["dim"]))
        self.fdesc.desc = desc
        desc = pref['fgrad']
        self.fgrad.setText(desc['des']+' | '+str(desc["dim"]))
        self.fgrad.desc = desc
        desc = pref['ftitle']
        self.ftitle.setText(desc['des']+' | '+str(desc["dim"]))
        self.ftitle.desc = desc
        self.fontbuttongroup.buttonClicked[int].connect(self.font_button_clicked)
        # Color group buttons
        self.buttonlist = [self.button1,
                           self.button2,
                           self.button3,
                           self.button4,
                           self.button5,
                           self.button6,
                           self.button7]
        self.buttongroup = QButtonGroup(parent)
        for id_, button in enumerate(self.buttonlist):
            self.buttongroup.addButton(button, id_)
        self.colorlist = [[int(x) for x in color[4:-1].split(',')] \
                           for color in pref['colors']]
        for index in range(len(self.buttonlist)):
            self.setcolor(index, self.colorlist[index])
        self.buttonBox.clicked['QAbstractButton *'].connect(self.restore_default)
        self.bindsgc()
        # Author info
        self.aname.setText(pref['aname'])
        self.amail.setText(pref['amail'])

        # Level group SpinButtons
        self.levlist = [self.lev_1,
                           self.lev_2,
                           self.lev_3,
                           self.lev_4,
                           self.lev_5,
                           self.lev_6,
                           self.lev_7]
        for i in range(len(pref['levels'])):
            self.levlist[i].setValue(float(pref['levels'][i]))
            self.levlist[i].editingFinished.connect(self.updatelevelconstraints)
        self.updatelevelconstraints()

    def restore_default(self, button):
        button_id = self.buttonBox.standardButton(button)
        if button_id == QDialogButtonBox.RestoreDefaults:
            self.olines.setChecked(bool(self.default['orizzontal lines']))
            self.showinfo.setChecked(bool(self.default['slopeinfo']))
            self.effect3d.setChecked(bool(self.default['effect-3d']))
            self.vcheck.setChecked(bool(self.default['latestVersionCheck']))
            res = 0
            if self.default['width'] == 1600:
                res = 3
            if self.default['width'] == 1280:
                res = 2
            elif self.default['width'] == 1024:
                res = 1
            self.res.setCurrentIndex(res)
            index = self.theme.findText(self.default['theme'])
            if index:
                self.theme.setCurrentIndex(index)
            self.serv.setCurrentIndex(0)
            if self.default['server'] == 'usgs.net':
                self.serv.setCurrentIndex(2)
            elif self.default['server'] == 'earthtools.org':
                self.serv.setCurrentIndex(1)

            self.fdesc.setText(self.default['font-des 1']+' | '+self.default['font-dim 1'])
            self.fdesc.desc = {
                                   "des": self.default['font-des 1'],
                                   "typ": self.default['font-typ 1'],
                                   "dim": self.default['font-dim 1']
                               }
            self.fgrad.setText(self.default['font-des g']+' | '+self.default['font-dim g'])
            self.fgrad.desc = {
                                   "des": self.default['font-des g'],
                                   "typ": self.default['font-typ g'],
                                   "dim": self.default['font-dim g']
                               }
            self.ftitle.setText(self.default['font-des t']+' | '+self.default['font-dim t'])
            self.ftitle.desc = {
                                   "des": self.default['font-des t'],
                                   "typ": self.default['font-typ t'],
                                   "dim": self.default['font-dim t']
                                }
            self.aname.setText(self.default['authorname'])
            self.amail.setText(self.default['authormail'])

            colorlist = [self.default["color "+str(i)] for i in range(1, 8)]
            self.colorlist = [list(map(int, color[4:-1].split(','))) \
                               for color in colorlist]
            for index in range(len(self.buttonlist)):
                self.setcolor(index, self.colorlist[index])

            levellist = [self.default["level "+str(i)] for i in range(1, 8)]
            for i in range(len(self.levlist)):
                # disable range fot this widget
                self.levlist[i].setMinimum(0.0)
                self.levlist[i].setMaximum(90.0)
                # set the range for this widget
                self.levlist[i].setValue(float(levellist[i]))
            self.updatelevelconstraints()

    def bindsgc(self):
        """Bind Setgcolor buttons """
        self.buttongroup.buttonClicked[int].connect(self.changecolor)

    def setcolor(self, index, color):
        """Set color in the colorlist"""
        qcolor = QColor(*color)
        palette = self.buttonlist[index].palette()
        palette.setColor(QPalette.Button, qcolor)
        self.colorlist[index] = (color[0], color[1], color[2])
        self.buttonlist[index].setStyleSheet("QWidget { background-color: %s }"
            % qcolor.name())

    def updatelevelconstraints(self):
        for i in range(1, len(self.levlist)):
            self.levlist[i-1].setMaximum(self.levlist[i].value()-0.1)
        for i in range(len(self.levlist)-1):
            self.levlist[i+1].setMinimum(self.levlist[i].value()+0.1)

    def changecolor(self, index):
        """Change the color used by plot to fill different classes of gradients
        """
        button = self.buttonlist[index]
        palette = button.palette()
        qcolor = palette.color(QPalette.Button)
        newcolor = QColorDialog.getColor(qcolor, self)
        if newcolor.isValid():
            self.setcolor(index,
                          (newcolor.red(),
                           newcolor.green(),
                           newcolor.blue()))

    def font_button_clicked(self, index):
        button = self.fontbuttonlist[index]
        qfont = QFont()
        qfont.setFamily(button.desc["des"])
        qfont.setPointSize(int(button.desc["dim"]))
        try:
            res = eval('QFont.'+button.desc["typ"].capitalize())
        except AttributeError:
            res = QFont.Normal
        qfont.setWeight(res)
        (qfont, ok) = QFontDialog.getFont(qfont, self)
        if ok:
            button.desc = {
                            "des": str(qfont.family()),
                            "typ": FONT_TYP[qfont.weight()],
                            "dim": str(qfont.pointSize())
                          }
            button.setText(str(qfont.family())+' | '+str(qfont.pointSize()))

    def getconfig(self):
        """Returns selected config options"""
        res = str(self.res.currentText()).split(' ')
        settings = {
                    'olines': self.olines.isChecked(),
                    'theme': str(self.theme.currentText()),
                    'sinfo': self.showinfo.isChecked(),
                    'legend': self.legend.isChecked(),
                    'vcheck': self.vcheck.isChecked(),
                    '3d': self.effect3d.isChecked(),
                    'aname': self.aname.text(),
                    'amail': self.amail.text(),
                    'fdesc': self.fdesc.desc,
                    'ftitle': self.ftitle.desc,
                    'fgrad': self.fgrad.desc,
                    'width': res[0],
                    'height': res[2],
                    'serv': str(self.serv.currentText()),
                    'colors': ['rgb(%d,%d,%d)' % (clr[0], clr[1], clr[2]) \
                                for clr in self.colorlist],
                    'levels': ['%f' % (lev.value()) for lev in self.levlist]
                   }
        if not VERSION_CHECK_ENABLED:
            del settings['vcheck']
        return settings

class Plot(QMainWindow):
    """ Plot window"""
    def __init__(self, parent, slope, plot_settings,
                 exportfunc, exportfunc_pdf, exportfunc_html):
        QMainWindow.__init__(self, parent)
        self.panel = AnimatedPanel(slope)
        self.setCentralWidget(self.panel)
        title = "CycloGraph"
        if slope.name:
            title += " - " + slope.name
        self.setWindowTitle(title)
        self.myslope = slope

        self.menu_item_save = QAction(_('Save'), self)
        icon_save = QIcon(QPixmap(":/qt-icons/pixmaps/save.png"))
        icon_save = QIcon.fromTheme("document-save-as", icon_save)
        self.menu_item_save.setIcon(icon_save)
        self.menu_item_savepdf = QAction(_('Save PDF'), self)
        self.menu_item_savehtml = QAction(_('Save html'), self)
        self.menu_item_olines = QAction(self)
        self.menu_item_olines.setCheckable(True)
        self.menu_item_olines.setChecked(plot_settings['olines'])
        self.menu_item_olines.setText(_("&Orizzontal lines"))
        self.menu_item_3d = QAction(self)
        self.menu_item_3d.setCheckable(True)
        self.menu_item_3d.setChecked(plot_settings['3d'])
        self.menu_item_3d.setText(_("3D &effect"))
        self.menu_item_legend = QAction(self)
        self.menu_item_legend.setCheckable(True)
        self.menu_item_legend.setChecked(plot_settings['legend'])
        self.menu_item_legend.setText(_("Show &legend"))

        self.themeitems = {}
        self.theme_action_group = QActionGroup(self)
        self.thememanager = ThemeManager()
        for theme in self.thememanager.getthemeslist():
            themeitem = QAction(self)
            themeitem.setCheckable(True)
            if plot_settings['theme'] == theme:
                themeitem.setChecked(True)
            themeitem.setText(theme)
            self.theme_action_group.addAction(themeitem)
            self.themeitems[themeitem] = theme

        menubar = self.menuBar()
        options_menu = menubar.addMenu(_('&Options'))
        options_menu.addAction(self.menu_item_save)
        options_menu.addAction(self.menu_item_savepdf)
        options_menu.addAction(self.menu_item_savehtml)
        options_menu.addAction(self.menu_item_olines)
        options_menu.addAction(self.menu_item_3d)
        options_menu.addAction(self.menu_item_legend)
        themes_menu = QMenu(options_menu)
        themes_menu.setTitle(_("&Theme"))
        for action in list(self.themeitems.keys()):
            themes_menu.addAction(action)
        options_menu.addAction(themes_menu.menuAction())
        res_menu = QMenu(options_menu)
        res_menu.setTitle(_("&Saving resolution"))
        self.resitems = {}
        res_action_group = QActionGroup(self)
        rlist = [('800', '600'), ('1024', '768'), ('1280', '800'), ('1600', '1200')]
        for res in rlist:
            resitem = QAction(self)
            resitem.setCheckable(True)
            restext = res[0]+'x'+res[1]
            resitem.setText(restext)
            if plot_settings['width'] == int(res[0]):
                resitem.setChecked(True)
            res_action_group.addAction(resitem)
            self.resitems[resitem] = restext
            res_menu.addAction(resitem)
        options_menu.addAction(res_menu.menuAction())
        self.exportfunc = exportfunc
        self.exportfunc_pdf = exportfunc_pdf
        self.exportfunc_html = exportfunc_html
        self.myslope.calc()
        self.settings = plot_settings
        self.panel.settings = self.settings
        self.panel.devc = glal.DeviceContext()
        self.resize(plot_settings['width'], plot_settings['height'])

        self.settings['colors'] = plot_settings['colors']

        self.animation3d = QtCore.QPropertyAnimation(self.panel, b"depth")
        self.animation3d.setDuration(1000)
        self.animation3d.setStartValue(100)
        self.animation3d.setEndValue(0)
        self.animationTh = QtCore.QPropertyAnimation(self.panel, b"opacity")
        self.animationTh.setDuration(1000)
        self.animationTh.setStartValue(1.0)
        self.animationTh.setEndValue(0.0)

        self.menu_item_save.triggered.connect(self.saveimage)
        self.menu_item_savepdf.triggered.connect(self.savepdf)
        self.menu_item_savehtml.triggered.connect(self.savehtml)
        self.menu_item_olines.triggered.connect(self.olines)
        self.menu_item_3d.triggered.connect(self.change3d)
        self.menu_item_legend.triggered.connect(self.changelegend)
        self.theme_action_group.triggered['QAction *'].connect(self.changetheme)
        res_action_group.triggered['QAction *'].connect(self.changeres)

    def showplot(self):
        """ Show plot"""
        self.show()

    def saveimage(self):
        """ Save image"""
        self.exportfunc(self.myslope, self.settings)

    def savepdf(self):
        """ Save pdf"""
        self.exportfunc_pdf(self.myslope, self.settings)

    def savehtml(self):
        """ Save html report"""
        self.exportfunc_html(self.myslope, self.settings)

    def change3d(self):
        """ If active switch 3d effect off, else turn it on."""
        if self.settings['3d']:
            self.animation3d.setDirection(self.animation3d.Forward)
        else:
            self.animation3d.setDirection(self.animation3d.Backward)
        self.animation3d.start()

    def changelegend(self):
        """ If active draws legend"""
        self.panel.shot()
        self.settings['legend'] = not self.settings['legend']
        self.animationTh.start()

    def olines(self):
        """ If active draws orizzontal lines"""
        self.panel.shot()
        self.settings['olines'] = not self.settings['olines']
        self.animationTh.start()

    def changetheme(self, action):
        """Change the actual slope theme."""
        self.panel.shot()
        self.settings["theme"] = self.themeitems[action]
        self.animationTh.start()

    def changeres(self, action):
        """Change saving resolution."""
        text = self.resitems[action]
        width, height = text.split('x')
        self.settings['width'] = int(width)
        self.settings['height'] = int(height)

class AnimatedPanel(QWidget):
    """This class implements a personalized widget that draws a slope.
    It also implements the fade between different images and 2D to 3D.
    Using this class prevents from drawing on the menu."""
    def __init__(self, slope, parent=None):
        QWidget.__init__(self, parent)
        self.__old_opacity = 0.0
        self.__depth = 100
        self.myslope = slope

    def shot(self):
        """Create an internal pixmap representing the current state
        of the widget"""
        self.__old_pixmap = QPixmap(self.size())
        self.render(self.__old_pixmap)

    def oldOpacity(self):
        """Get the opacity of the pixamp of the old slope"""
        return self.__old_opacity

    def setOldOpacity(self, opacity):
        """Set the opacity of the pixamp of the old slope"""
        self.__old_opacity = opacity
        self.repaint()

    def changeres(self, action):
        """Change saving resolution."""
        text = self.resitems[action]
        width, height = text.split('x')
        self.settings['width'] = int(width)
        self.settings['height'] = int(height)
    opacity = QtCore.pyqtProperty("double", oldOpacity, setOldOpacity)

    def depth(self):
        """Get the depth of the current slope (0 means 2D)"""
        return self.__depth

    def setDepth(self, depth):
        """Set the depth of the current slope"""
        self.__depth = depth
        self.settings['3d'] = (depth != 0)
        self.repaint()
    depth = QtCore.pyqtProperty("int", depth, setDepth)

    def paintEvent(self, event):
        """Actually draws the content."""
        self.myslope.depth = self.__depth
        self.devc.init_draw_surf(self)
        self.myslope.paint(self.settings, self.devc)
        if self.__old_opacity > 0.0:
            self.devc.paint.resetTransform()
            self.devc.paint.setOpacity(self.__old_opacity)
            self.devc.paint.drawPixmap(0, 0, self.__old_pixmap)
        self.devc.end_draw()

class Map(QDialog, ui_qt.Ui_map):
    """Show map of slopes"""
    def __init__(self, parent, slopename, stringc, savefnct):
        QDialog.__init__(self, parent)
        self.setupUi(self)
        self.savefnct = savefnct
        self.setWindowFlags(QtCore.Qt.Window)
        self.setWindowTitle(_("Map") + " - " + slopename)
        self.buttonBox.clicked['QAbstractButton *'].connect(self.button_clicked)
        mapfile = QtCore.QFile(":/openlayers/cyclograph/map.html")
        if mapfile.open(QtCore.QFile.ReadOnly):
            mapdata = bytes(mapfile.readAll()).decode()
            htmlMap = mapdata % (stringc,)
            mapfile.close()
            self.webView.setHtml(htmlMap, QtCore.QUrl('file://'))
        self.show()

    def saveimage(self, filepath):
        """ Save map image"""
        page = self.webView.page()
        img = QImage(page.contentsSize().toSize(), QImage.Format_ARGB32)
        painter = QPainter(img)
        self.webView.render(painter)
        painter.end()
        img.save(filepath)

    def button_clicked(self, button):
        button_id = self.buttonBox.standardButton(button)
        if button_id == QDialogButtonBox.Save:
            self.savefnct(self)

class Create_kml(QDialog, ui_qt.Ui_Create_KML_Dialog):
    """Create kml dialog"""
    MODE = {_("Open Route Service"): 'ORS',
            _(u'brouter'): 'BROU',
            _('yournavigation'): 'YOURS',
            _("Draw on the map"): 'DRAW'}

    def __init__(self, parent, downloadfunc, savefunc):
        QDialog.__init__(self, parent)
        self.setupUi(self)
        self.setWindowFlags(QtCore.Qt.Window)
        self.downloadfunc = downloadfunc  # used to downlaod from url
        self.savefunc = savefunc
        self.content = None
        self.mode = None
        self.mode_changed(self.listWidget.currentItem().text())
        self.splitter.setStretchFactor(0, 1)
        self.splitter.setStretchFactor(1, 3)

        self.buttonBox.clicked['QAbstractButton *'].connect(self.button_clicked)
        self.listWidget.currentTextChanged['const QString&'].connect(self.mode_changed)

    def webview_init(self):
        """ Check mode and set webview accordingly"""
        if self.mode == 'ORS':
            self.webView.setUrl(QtCore.QUrl("http://openrouteservice.org/"))
        elif self.mode == 'BROU':
            self.webView.setUrl(QtCore.QUrl("http://brouter.de/brouter-web/"))
        elif self.mode == 'YOURS':
            self.webView.setUrl(QtCore.QUrl("http://yournavigation.org/"))
        elif self.mode == 'DRAW':
            url = QtCore.QUrl("qrc:/openlayers/cyclograph/openlayers.html")  # .fromLocalFile(os.path.join(os.path.dirname(__file__),"openlayers.html"))
            self.webView.setUrl(url)

    def button_clicked(self, button):
        button_id = self.buttonBox.standardButton(button)
        if button_id == QDialogButtonBox.Ok:
            self.import_kml()
        elif button_id == QDialogButtonBox.Save:
            self.save_kml()
        elif button_id == QDialogButtonBox.Cancel:
            self.content = None
            #Close the dialog immediately
            QDialog.reject(self)

    def mode_changed(self, sel_string):
        """ Change current mode"""
        newmode = self.MODE[str(sel_string)]
        if newmode != self.mode:
            self.mode = newmode
            self.webview_init()

    def get_ors(self, callback):
        page = self.webView.page()

        #jsresult = frame.evaluateJavaScript("document.getElementById('gpx').innerHTML")
        def jscallback(jsresult):
            if jsresult is None:
                self.content = None
                return
            gpxcontent = str(jsresult)
            from xml.sax.saxutils import unescape
            self.content = unescape(gpxcontent)
            callback()
        page.runJavaScript("document.getElementById('gpx').innerHTML", jscallback) #FIXME: this is not working: getElementById returns null

    def get_brou(self, callback):
        """ Returns kml content from BROU service"""
        page = self.webView.page()

        def jscallback(jsresult):
            if jsresult is None:
                self.content = None
                return
            url = str(jsresult)
            handler = self.downloadfunc(url)
            self.content = handler.read().decode('utf-8')
            callback()
        page.runJavaScript("$(\".value\").find('a:contains(\"GPX\")')[0].href", jscallback)

    def get_yours(self, callback):
        """ Returns kml content from YOURS service"""
        page = self.webView.page()

        def jscallback(jsresult):
            if jsresult is None:
                self.content = None
                return
            elemtext = str(jsresult)
            text = elemtext.split('href="http://yournavigation.org/?')
            turl = "http://www.yournavigation.org/api/1.0/gosmore.php?format=kml&"
            if len(text) > 1:
                turl += text[1].split("\">")[0]
                turl = turl.replace("&amp;", "&")
            handler = self.downloadfunc(turl)
            self.content = handler.read().decode('utf-8')
            callback()
        page.runJavaScript("document.getElementById('feature_info').innerHTML", jscallback)

    def get_draw(self, callback):
        page = self.webView.page()

        def jscallback(jsresult):
            self.content = jsresult
            callback()
        page.runJavaScript("createKml()", jscallback)

    def download(self, callback):
        if self.mode == 'ORS':
            self.get_ors(callback)
        elif self.mode == 'BROU':
            self.get_brou(callback)
        elif self.mode == 'YOURS':
            self.get_yours(callback)
        elif self.mode == 'DRAW':
            self.get_draw(callback)

    def import_kml(self):
        def import_callback():
            QDialog.accept(self)

        self.download(import_callback)

    def save_kml(self):
        def download_callback():
            if not self.content:
                return
            self.savefunc(self.content, self.mode)
            # Do not accept the dialog: it must stay open if the user wants to
            # import without opening the file saved or if he hasn't saved and
            # wants to go back.

        self.download(download_callback)


    def show_modal(self):
        """Shows the dialog and returns True if the value has to be chenged."""
        val = self.exec_()
        return val == QDialog.Accepted

class NumcpsDialog(QDialog, ui_qt.Ui_numcps_dlg):
    """ask cps number you want to import"""
    def __init__(self, max_num_cps, start_dist, finish_dist, file_ext):
        QDialog.__init__(self, None)
        self.setupUi(self)
        self.slider.setMaximum(max_num_cps)
        self.spinBox.setMaximum(max_num_cps)
        self.start.setRange(start_dist, finish_dist)
        self.start.setValue(start_dist)
        self.end.setRange(start_dist, finish_dist)
        self.end.setValue(finish_dist)
        selrange = self.end.value() - self.start.value()
        # the value of the slider_2 is multiplied by 1000 to have a resolution of 0.001
        self.slider_2.setMaximum(selrange*1000)
        self.doubleSpinBox_2.setMaximum(selrange)
        text = _("The %s file contains %d points.\nChoose how many will be imported.") % (file_ext, max_num_cps)
        self.label.setText(text)
        self.file_ext = file_ext
        self.label_2.setText(_("The %s file contains a route of %.3f Km.\nChoose what range you want between points.") % (file_ext, finish_dist))
        self.label_3.setText(_("Choose the type of import you want"))

        self.proceedButton.clicked.connect(self.onProceedClicked)
        self.start.valueChanged.connect(self.updateRange)
        self.end.valueChanged.connect(self.updateRange)
        self.slider_2.valueChanged.connect(self.updateSlider_2)
        self.doubleSpinBox_2.valueChanged.connect(self.updateRange_2)

    def onProceedClicked(self):
        btn = self.buttonGroup.checkedButton()
        if btn == self.all_ or btn == self.automatic:
            self.accept()
        elif btn == self.number:
            self.stackedWidget.setCurrentIndex(1)
        elif btn == self.distance_2:
            self.stackedWidget.setCurrentIndex(2)

    def updateRange(self):
        "The range changed"
        self.start.setMaximum(self.end.value() - self.end.singleStep())
        self.end.setMinimum(self.start.value() + self.start.singleStep())
        self.slider_2.setMaximum((self.end.value() - self.start.value())*1000)
        self.doubleSpinBox_2.setMaximum(self.end.value() - self.start.value())

    def updateRange_2(self):
        "The range_2 changed"
        spinval = self.doubleSpinBox_2.value()
        self.slider_2.setValue(int(spinval*1000))

    def updateSlider_2(self):
        "The Slider_2 value changed"
        sliderval = self.slider_2.value()/1000.0
        self.doubleSpinBox_2.setValue(sliderval)


def preferences(parent, cgpref, defaultpref):
    """ set Preference dialog"""
    dlg = Preferences(parent, cgpref, defaultpref)
    if dlg.exec_() == QDialog.Rejected:
        return {}
    else:
        return dlg.getconfig()


def formslopeinfo(parent, slopedata):
    """ Show slope's info"""
    dlg = FormSlopeInfo(parent, slopedata)
    if dlg.exec_() == QDialog.Rejected:
        return {}
    else:
        return dlg.getslopeinfo()


def FileDialog(parent, title, dirname, filename, other, file_types, rw, FD_OPEN, FD_SAVE):
    """ Show File dialog"""
    filters = reduce(lambda x, y: x + ';;' + y,
                      [("%s (%s)" % elem) for elem in file_types])
    # print filters
    if rw == FD_OPEN:
        result = QFileDialog.getOpenFileName(parent,  title,
                                                 dirname,  filters)
    elif rw == FD_SAVE:
        result = QFileDialog.getSaveFileName(parent,  title,
                                                 dirname,  filters)
    if len(result) == 2:
        #result is a tuple (filename, selected_file_option)
        return str(result[0])
    else:
        return None


def numcpsdlg(max_num_cps, start_dist, finish_dist, file_ext):
    """ask how many cps in a gpx or tcx files shall be imported"""
    dlg = NumcpsDialog(max_num_cps, start_dist, finish_dist, file_ext)
    if dlg.exec_() == QDialog.Rejected:
        return (-1, 0, start_dist, finish_dist)
    elif dlg.buttonGroup.checkedButton() == dlg.automatic:
        return (0, -1, dlg.start.value(), dlg.end.value())
    elif dlg.buttonGroup.checkedButton() == dlg.all_:
        return (0, max_num_cps, dlg.start.value(), dlg.end.value())
    elif dlg.stackedWidget.currentIndex() == 1:
        return (0, dlg.slider.value(), dlg.start.value(), dlg.end.value())
    else:
        value = dlg.doubleSpinBox_2.value()
        return (1, value, dlg.start.value(), dlg.end.value())


def geterrnoroute(parent):
    """Get an error message explaining that you should have selected one
       route"""
    return QMessageBox.warning(parent, _('No route'), _("Specify one route first."),
                                        QMessageBox.Ok)


def managecperr(parent):
    """Get an error message explaining that you should have inserted
       altitude and distance"""
    return QMessageBox.critical(parent, _("Form incomplete"),
                                      _("Distance and altitude are required."),
                                      QMessageBox.Ok)


def mapcoorderr(parent):
    """Get an error message explaining that there are no coords in the slope"""
    return QMessageBox.critical(parent, _("No coordinates"),
                                      _("there aren't coordinates associated with this slope."),
                                      QMessageBox.Ok)


def geterrnocp(parent):
    """Get an error message explaining that you should have at least two cps
       to plot
    """
    return QMessageBox.warning(parent, _('Not enough check points'), _(
"""There aren't enougth check points to plot.
 At least two points are required to plot."""),
                                        QMessageBox.Ok)


def save_changes(parent, filename):
    """Show a message if you close a modified and not saved slope"""
    response = None
    SAVE = _('Save')
    DISCARD = _('Close without saving')
    CANCEL = _('Cancel')
    message = QMessageBox(parent)
    message.setText(_("Save changes to %s before closing?") % filename)
    message.setWindowTitle(_('Save changes'))
    message.setIcon(QMessageBox.Warning)
    save    = message.addButton(SAVE,    QMessageBox.AcceptRole)
    discard = message.addButton(DISCARD, QMessageBox.DestructiveRole)
    cancel  = message.addButton(CANCEL,  QMessageBox.RejectRole)
    message.exec_()
    response = message.clickedButton()
    if response == save:
        return 'SAVE'
    if response == discard:
        return 'DISCARD'
    if response == cancel:
        return 'CANCEL'
    return response

# vim:sw=4:softtabstop=4:expandtab



# -*- coding: utf-8 -*-
# qt.glal.py
"""This module provides a graphic library abstraction layer for Cyclograph"""

# Copyright (C) 2008-2014 Federico Brega, Pierluigi Villani

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.



from PyQt5.QtGui import *
from PyQt5.QtWidgets import *
from PyQt5.QtPrintSupport import QPrinter
from PyQt5 import QtCore
from version import VERSION

from . import gui


def initapp(*args):
    """Create initial app for gui"""
    app = QApplication(*args)
    
    # Tell to the graphic library witch language use
    translations_path = QtCore.QLibraryInfo.location(QtCore.QLibraryInfo.TranslationsPath)
    translation_file = "qt_" + str(QtCore.QLocale.system().name())
    translator = QtCore.QTranslator(app)
    translator.load(translation_file, translations_path)
    app.installTranslator(translator)
    
    #Enable support for High resolution disaplays
    app.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)
    
    return app


def bind(cgfunc, menuopt):
    """Connect gui menu option with cg function"""
    return menuopt.triggered.connect(cgfunc)


def bind_close(cg_exit, gui):
    """Connect gui close event with cg exit function"""
    gui.cg_exit = cg_exit
    return gui.menu_item_exit.triggered.connect(gui.quit)


def ToolbarBind(fun, button):
    """Connect gui toolbar button with cg function"""
    return button.triggered.connect(fun)


def OptionCheck(guiopt, option):
    """Set a gui menu option to selected/unselected """
    guiopt.setChecked(option)


def enable_saving(gui, bool_val):
    """Enable or disable saving options according to bool_val"""
    gui.menu_item_save.setEnabled(bool_val)
    gui.menu_item_save_as.setEnabled(bool_val)
    gui.action_add.setEnabled(bool_val)
    gui.action_edit.setEnabled(bool_val)
    gui.action_delete.setEnabled(bool_val)
    gui.action_plot.setEnabled(bool_val)
    gui.action_map.setEnabled(bool_val)
    gui.action_properties.setEnabled(bool_val)


def addstatusbartext(maingui, text):
    """Add text to main gui status bar"""
    maingui.addstatusbartext(text)


def signalbug(event=None):
    """Open webpage to signal bugs"""
    bugUrl = 'http://sourceforge.net/project/memberlist.php?group_id=227295'
    QDesktopServices.openUrl(QtCore.QUrl(bugUrl))

class Message(QtCore.QObject):
    """ Qt message"""
    
    sent = QtCore.pyqtSignal(str, int, int) #message name, slope_number, row_num
    
    def __init__(self):
        QtCore.QObject.__init__(self)
    
    def send(self, message, slope_number, row_num):
        """ Send message"""
        self.sent.emit(message, slope_number, row_num)
        
    @staticmethod
    def subscribe(function, emitter, message):
        """ Subscribe message"""
        def callback_check(message_sent, slope_number, row_num):
            if message_sent == message:
                function(slope_number, row_num)
        emitter.sent.connect(callback_check)


class Notebook:
    """ Qt Notebook"""
    def __init__(self, notebook, fun_close):
        self.notebook = notebook
        self.fun_close = fun_close
        self.notebook.setTabsClosable(True)
        self.notebook.tabCloseRequested[int].connect(fun_close)

    def Page(self):
        """ Return qt Page"""
        return gui.Page(self.notebook)

    def set_page_label(self, page_num, text):
        """ Set page label text"""
        self.notebook.setTabText(page_num,  text)

    def add_page(self, page, title):
        """ Add page"""
        self.notebook.addTab(page, title)
        page.button_close.setVisible(False)
        self.notebook.setCurrentWidget(page)

    def remove_page(self, page_num):
        """ Remove page"""
        self.notebook.removeTab(page_num)

    def getselpagnum(self):
        """ Return number from selected page"""
        return self.notebook.currentIndex()

    def setselpagnum(self, page_num):
        """Set current selected page"""
        self.notebook.setCurrentWidget(self.get_page(page_num))

    def get_page(self, page_num):
        """ Return page"""
        page = self.notebook.widget(page_num)
        return page

    def get_pagenum(self, argument):
        """Return page_num from page passed as argument"""
        return argument[0]


class DeviceContext():
    """Device Context Qt class wrapper"""
    def _convqcolor(self, color):
        """ Convert color"""
        if isinstance(color, str):
            if color.startswith("rgb"):
                col = list(map(int, color[4:-1].split(',')))
                return QColor(*col)
            else:
                return QColor(color)
        else:
            return QColor(*color)

    def init_draw_surf(self, panel):
        """ Initialize drawing surface"""
        (self.size_x, self.size_y) = (panel.width(), panel.height())
        self.paint = QPainter()
        self.paint.begin(panel)
        self.paint.setRenderHint(QPainter.SmoothPixmapTransform)
        self.paint.setRenderHint(QPainter.Antialiasing)

    def shear(self, shear):
        """ device shear"""
        self.paint.shear(0.0, shear)

    def getsize(self):
        """ Return size"""
        return (self.size_x, self.size_y)

    def gradfill(self, rect, startcolor, endcolor):
        """ Fill gradient"""
        (sx, sy, ex, ey) = (rect[0], rect[1], rect[0], rect[1]+rect[3])
        gradient = QLinearGradient(sx, sy, ex, ey)
        sqcol = self._convqcolor(startcolor)
        gradient.setColorAt(0, sqcol)
        eqcol = self._convqcolor(endcolor)
        gradient.setColorAt(1, eqcol)
        brush = QBrush(gradient)
        self.paint.fillRect(QtCore.QRect(*rect), brush)

    def setpen(self, color, size):
        """ Set pen's color and size"""
        qcol = self._convqcolor(color)
        pen = QPen(qcol)
        pen.setWidth(size)
        self.paint.setPen(pen)

    def setfont(self, fontdict):
        """ Set font to use"""
        font = QFont()
        self.fontmetrics = QFontMetrics(font)
        try:
            font.setFamily(fontdict["des"])
            font.setPointSize(fontdict["dim"])
            font.setWeight(eval('QFont.'+fontdict["typ"].capitalize()))
            self.paint.setFont(font)
        except Exception:
            pass

    def drawtext(self, text, pos_x, pos_y):
        """ Draw text at position x,y"""
        self.paint.drawText(pos_x, pos_y+10, text)

    def gettextwidth(self, text):
        """ Return text length"""
        return self.fontmetrics.width(text)

    def gettextheight(self, text):
        """ Return text height"""
        return self.fontmetrics.height()

    def drawline(self, pos_x0, pos_y0, pos_x1, pos_y1):
        """ Draw line"""
        self.paint.drawLine(pos_x0, pos_y0, pos_x1, pos_y1)

    def setlineargradientbrush(self, colorlist, startp, endp):
        """ Get a linear gradient from startp to endp, using colors in colorlist.
        The elments of colorlist are tuple in the format (color, realtive position)."""
        grad = QLinearGradient(startp[0], startp[1], endp[0], endp[1])
        for color in colorlist:
            grad.setColorAt(color[1], self._convqcolor(color[0]))
        brush = QBrush(grad)
        self.paint.setBrush(brush)

    def setbrush(self, color):
        """ Set brush's color"""
        qcol = self._convqcolor(color)
        brush = QBrush(qcol)
        self.paint.setBrush(brush)

    def drawrectangle(self, pos_x0, pos_y0, width, height):
        """ Draw rectangle"""
        self.paint.drawRect(pos_x0, pos_y0, width, height)

    def drawrotatedtext(self, text, pos_x, pos_y, angle):
        """ Draw rotated text at position x,y by angle"""
        self.paint.save()
        self.paint.rotate(-angle)
        transform = QTransform()
        transform.rotate(angle)
        (newx, newy) = transform.map(pos_x+10, pos_y-5)
        self.paint.drawText(newx, newy, text)
        self.paint.restore()

    def drawpolygon(self, sequence):
        """ Draw polygon"""
        points = [QtCore.QPoint(*p) for p in sequence]
        self.paint.drawPolygon(*points)

    def startpath(self, point):
        """ Start a path in the specified point,"""
        point = QtCore.QPointF(*point)
        path = QPainterPath()
        path.moveTo(point)
        return path

    def drawpathlineto(self, path, point):
        """ Draw a straight line from the last point to the given point."""
        point = QtCore.QPointF(*point)
        path.lineTo(point)

    def drawpathcubicto(self, path, controlpoints):
        """ Draw a cubic Beziér frome the last point using the given list of
        three control points."""
        points = [QtCore.QPointF(pnt[0], pnt[1]) for pnt in controlpoints]
        path.cubicTo(*points)

    def endpath(self, path):
        """ Show the path."""
        self.paint.drawPath(path)

    def end_draw(self):
        """ Finish drawing"""
        self.paint.end()
        del self.paint


class Image():
    """Image Qt class wrapper"""
    def __init__(self, size_x,  size_y, plotfnct):
        (self.size_x, self.size_y) = (size_x, size_y)
        self.plotfnct = plotfnct
        self.image = QPixmap(size_x, size_y)
        self.image.fill()  # white backgound

    def plot(self, settings):
        """ Draw slope to the device context"""
        dcwrpp = DeviceContext()
        (dcwrpp.size_x, dcwrpp.size_y) = (self.size_x, self.size_y)
        dcwrpp.paint = QPainter()
        dcwrpp.paint.begin(self.image)
        dcwrpp.paint.setRenderHint(QPainter.SmoothPixmapTransform)
        dcwrpp.paint.setRenderHint(QPainter.Antialiasing)
        self.plotfnct(settings, dcwrpp)
        dcwrpp.end_draw()

    def savetofile(self, path, format):
        """ Save image to file"""
        self.image.save(path, format)


class Pdf():
    """Qt pdf class wrapper"""
    def __init__(self, filepath):
        self.dy = 90
        self.dx = 300
        self.y_incr = 30
        self.printer = QPrinter(QPrinter.ScreenResolution)
        self.printer.setOutputFormat(QPrinter.PdfFormat)
        self.printer.setOutputFileName(filepath)
        self.printer.setFullPage(True)
#        paperSize = QtCore.QSizeF(793, 1122)
#        self.printer.setPaperSize(paperSize, QPrinter.Millimeter)
        self.printer.setCreator('Cyclograph {0}'.format(VERSION))
        self.painter = QPainter()
        self.painter.begin(self.printer)
        self.painter.setRenderHint(QPainter.SmoothPixmapTransform)
        self.painter.setRenderHint(QPainter.Antialiasing)
        self.painter.setFont(QFont('Decorative', 18))

    def plot_image(self, settings, size_x, size_y, plotfnct):
        """ Draw slope to pdf"""
        image = QPicture()
        dcwrpp = DeviceContext()
        (dcwrpp.size_x, dcwrpp.size_y) = (size_x, size_y)
        dcwrpp.paint = QPainter()
        dcwrpp.paint.begin(image)
        dcwrpp.paint.setRenderHint(QPainter.SmoothPixmapTransform)
        dcwrpp.paint.setRenderHint(QPainter.Antialiasing)
        plotfnct(settings, dcwrpp)
        dcwrpp.end_draw()
        self.painter.drawPicture(0, 0, image)
        self.dy += size_y

    def addtext(self, text):
        self.dy += self.y_incr
        self.painter.setPen(QColor(0, 0, 0))
        self.painter.drawText(self.dx, self.dy, text)

    def addtitle(self, text):
        self.dy += self.y_incr
        self.painter.setPen(QColor(0, 0, 102))
        self.painter.drawText(self.dx-50, self.dy, text+":")

    def save(self):
        """Save pdf"""
        self.painter.end()


class ProgressDialog():
    """ Progress dialog for importkml"""
    def __init__(self):
        self.pd = QProgressDialog(_("Downloading altitudes"),
                                         _("Cancel"), 0, 1000)

    def update(self, value):
        """Update the progress shown and return if user want to abort."""
        self.pd.setValue(value)
        return self.pd.wasCanceled()

    def destroy(self):
        """ Destroy progress dialog"""
        self.pd = None


class Timer():
    """ Timer for Qt gui"""

    def __init__(self, period, callback):
        self.timer = self.timer = QtCore.QTimer()
        self.period = period
        self.timer.timeout.connect(callback)

    def start(self):
        """ Start timer count"""
        self.timer.start(self.period)

    def stop(self):
        """ Stop timer count"""
        self.timer.stop()

# vim:sw=4:softtabstop=4:expandtab



## imports are included from imports.py ##

from PyQt5 import QtCore, QtGui, QtWidgets, QtWebEngineWidgets
from . import cyclograph_rc

def cg_translate(context, text, **kwargs):
    return _(text)

## end of imports ##



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""glal_selected.py """

# Copyright (C) 2011 Federico Brega

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

import os.path
import sys

INTERFACES = {'qt': 'qt',
              'gtk3': 'gtk3',
              'cli': 'glal.py'}
if_path = os.path.join(os.path.dirname(os.path.abspath(__file__)))


def use(lib):
    """Set lib to use"""
    global library

    if not (lib in INTERFACES and os.path.exists(os.path.join(if_path, INTERFACES[lib]))):
        sys.stderr.write(lib + ' interface not installed\n')
        return None
    library = lib
    return library


def getname():
    """Return name of the used interface"""
    return library

# vim:sw=4:softtabstop=4:expandtab



# -*- coding: utf-8 -*-
# slope.py
"""This module provides a model for Cyclograph"""

# Copyright (C) 2008-2016 Federico Brega, Pierluigi Villani

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

from __future__ import unicode_literals
import math
import glal
from themes import ThemeManager


class Slope:
    """Model of a slope"""
    def __init__(self):
        """Create a slope model."""
        self.cps = []
        self.coords = []
        self.grad = []
        self.dercp = []
        self.name = ''
        self.country = ''
        self.author = ''
        self.email = ''
        self.comment = ''
        self.url = ''
        self.reset_calculated()

    def __len__(self):
        return len(self.cps)

    def reset_calculated(self):
        self.average_grad = 0
        self.max_grad = 0
        self.height_difference = 0
        self.height_gain = 0

    def add_cp(self, distance, altitude, name):
        """ Adds a check-point to a Slope.
        A check point must have an altitude and a distance from start,
       it can have also a name or description.
        """
        rdistance = float('%.3f'%(distance))
        new_cp = (rdistance, altitude, name)
        # check if there is already a cp with the same distance
        # and in this case remove it
        # WARNING: when loading from a file this cause complexity to be O(|#cps|^2)
        # If slowliness occurs consider using a binary search instead.
        for i in range(len(self.cps)):
            if self.cps[i][0] == rdistance:
                del self.cps[i]
                break
        self.cps.append(new_cp)
        self.cps.sort()
        self.grad = []
        self.reset_calculated()
        return self.cps.index(new_cp)

    def add_coord(self, latitude, longitude):
        """ Adds a coordinate to a Slope.
        A coordinate must have latitude and longitude.
        """
        new_coord = (latitude, longitude)
        self.coords.append(new_coord)

    def remove_cp(self, num):
        """ Removes check-point num from current slope and clears data."""
        del self.cps[num]
        self.grad = []
        self.reset_calculated()

    def shift(self, shift_amount):
        for i in range(len(self.cps)):
            self.cps[i][0] += shift_amount

    def calc(self):
        """ Update gradient and altitude bounds"""
        if len(self.cps) > 1:
            cps = self.cps[:]  # Multithread safe?
            derivate = lambda p0, p1: (p1[1] - p0[1])/(p1[0] - p0[0])
            self.grad = [derivate(cps[i], cps[i+1])/10 for i in range(len(cps)-1)]
            self.average_grad = 0
            self.max_grad = self.grad[0]
            for i in range(len(self.grad)):
                if self.grad[i] > self.max_grad:
                    self.max_grad = self.grad[i]
            self.height_gain = 0
            for i in range(len(cps)-1):
                self.height_gain += max(cps[i+1][1]-cps[i][1], 0)
            # find max & min altitude
            # float("inf") doesn't work on Windows
            self.max_h = -float("1e1000")
            self.min_h = +float("1e1000")
            for cpi in self.cps:
                if cpi[1] > self.max_h:
                    self.max_h = cpi[1]
                if cpi[1] < self.min_h:
                    self.min_h = cpi[1]
            self.height_difference = self.max_h - self.min_h
            self.max_h += 100
            self.min_h = int(math.floor(self.min_h/100)) * 100
            # min distance is always in the first item
            self.min_d = self.cps[0][0]
            # max distance is always in the last item
            self.max_d = self.cps[-1][0]

            if self.max_d != 0:
                self.average_grad = (self.cps[-1][1] - self.cps[0][1]) / (self.max_d * 10)

            self.dercp = self.smooth()

    def smooth(self):
        # References:
        #     Subroutine PCHIM, F. N. Fritsch, Lawrence Livermore National Laboratory.
        #     F. N. Fritsch and J. Butland, "A method for constructing local monotone
        #     piecewise cubic interpolants", SIAM J. Sci. Stat. Comput., vol. 5,
        #     pp. 300-304, June 1984.

        if not self.grad:
            return []
        if len(self.cps) < 3:
            # If less than 3 points draw a rect
            return [self.grad[0] * 10] * 2
        der = [0] * len(self.cps)

        # Inspired by Octave code in dpchim.f
        grad1 = self.grad[0] * 10
        grad2 = self.grad[1] * 10
        (h1, h2) = (self.cps[1][0] - self.cps[0][0], self.cps[2][0] - self.cps[1][0])
        w1 = (2 * h1 + h2) / (h1 + h2)
        w2 = -h1 / (h1 + h2)
        der[0] = w1 * grad1 + w2 * grad2
        if der[0] * grad1 <= 0:
            der[0] = 0
        elif grad1 * grad2 < 0:
            dmax = 3 * grad1
            if abs(der[0]) > abs(dmax):
                der[0] = dmax

        # Using brodlie modification of butland's formula
        for i in range(len(self.cps)-2):
            (h1, h2) = (float(self.cps[i][0] - self.cps[i-1][0]), float(self.cps[i+1][0] - self.cps[i][0]))
            grad1 = self.grad[i] * 10
            grad2 = self.grad[i+1] * 10
            if grad1 * grad2 <= 0:
                der[i] = 0
                continue
            dmax = max(abs(grad1), abs(grad2))
            dmin = min(abs(grad1), abs(grad2))
            w1 = (2 * h1 + h2) / (3 * (h1 + h2))
            w2 = (2 * h2 + h1) / (3 * (h1 + h2))
            der[i+1] = dmin / (w1 * grad1 / dmax + w2 * grad2 / dmax)

        grad1 = self.grad[-2] * 10
        grad2 = self.grad[-1] * 10
        (h1, h2) = (self.cps[1][0] - self.cps[0][0], self.cps[2][0] - self.cps[1][0])
        w1 = - h2 / (h1 + h2)
        w2 = (2 * h2 + h1) / (h1 + h2)
        der[-1] = w1 * grad1 + w2 * grad2
        if der[-1] * grad2 <= 0:
            der[-1] = 0
        elif grad1 * grad2 < 0:
            dmax = 3.0 * grad2
            if abs(der[-1]) > abs(dmax):
                der[-1] = dmax
        return der

    depth = 100

    def paint(self, settings, devc):
        """ Paint devc from plot"""
        # upper, lower, right and left margin of area where draw the slope
        theme = ThemeManager().gettheme(settings['theme'])
        updownmar = (180, 30)
        leftrightmar = (50, 10)
        margins = (updownmar, leftrightmar)
        (upp_mar, low_mar) = updownmar
        (lef_mar, rig_mar) = leftrightmar

        theme.paintbackground(devc, devc.size_x, devc.size_y)

        (max_x, max_y) = devc.getsize()
        if settings['3d']:
            devc.shear(theme.shear*self.depth/100)
            min_y = max_x*self.depth/100/10
            rig_mar = 10 + 20*self.depth/100
        else:
            min_y = 0

        theme.gradback(devc, max_x, max_y, settings['fdesc'])

        # draw altitude bar
        metersize = (max_y - upp_mar - low_mar - min_y) \
                    / (self.max_h - self.min_h)
        self.h_incr = 100  # draw a line every 100m

        if settings['olines']:
            theme.olines(devc, self, margins, max_x, max_y, metersize)

        theme.alttext(devc, self, margins, max_y, metersize)
        theme.yaxis(devc, self, margins, max_y, metersize)

        if settings['3d']:
            (dx, dy) = (20*self.depth/100, 10*self.depth/100)
        else:
            (dx, dy) = (0, 0)
        # draw km bar
        devc.setpen('black', 1)
        increments = [1, 2, 5, 10, 20, 50, 100]  # km bar resolutions
        for d_incr in increments:
            # draw less than 30 bars
            if (self.max_d - self.min_d) <= 30 * d_incr:
                break
        # this must be float otherwise there are problems with long slopes
        kmsize = (max_x - lef_mar - rig_mar) / (self.max_d - int(self.min_d))

        theme.xaxis(devc, self, margins, max_y, d_incr, kmsize, dx, dy)

        # draw slope's name
        s_info = (self.name,
                  _("Average gradient:")+" "+"%.1f" % self.average_grad+" %",
                  _("Max gradient:")+" "+"%.1f" % self.max_grad+" %",
                  _("Height difference:")+" "+str(self.height_difference)+" m",
                  _("Height gain")+": "+str(self.height_gain)+" m")
        theme.drawslopeinfo(devc, s_info, settings, lef_mar+20, min_y+upp_mar-140)

        # draw slope's legend
        if settings['legend']:
            theme.drawlegend(devc, settings, max_x-rig_mar-410, min_y+upp_mar-110, _("LEGEND"))

        # draw first info text
        font = settings['fdesc']
        theme.desctext(devc, "%.0f %s" % (self.cps[0][1], self.cps[0][2]),
                     lef_mar + int(self.cps[0][0] * kmsize) + 3,
                     max_y-low_mar-10-int((self.cps[0][1] - self.min_h) * metersize),
                     font)

        # plot the slope
        # plot orizzontal polygon in reverse order to prevent bad visualization in 3d mode
        if (dx != 0) and (dy != 0):
            linkpoints = []
            spath_back = []
            colorlisth = []
            for i in range(len(self.cps)-1):
                # i = len(self.cps)-1 - k
                v_a = (int((self.cps[i][0] - int(self.min_d)) * kmsize),
                       int((self.cps[i][1] - self.min_h) * metersize))
                v_b = (int((self.cps[i+1][0] - int(self.min_d)) * kmsize),
                       int((self.cps[i+1][1] - self.min_h) * metersize))

                points = [(lef_mar + v_a[0], max_y - low_mar - v_a[1]),
                        (lef_mar + v_b[0], max_y - low_mar - v_b[1]),
                        (lef_mar + v_b[0] + dx, max_y - low_mar - v_b[1] + dy),
                        (lef_mar + v_a[0] + dx, max_y - low_mar - v_a[1] + dy)]
                linkpoints.append(points)
                spath_back.append(polytoBezier(points[0],
                  self.dercp[i] * (-metersize / kmsize),
                  points[1],
                  self.dercp[i+1] * (-metersize / kmsize)))
            # theme.fillhslopecontour(devc, spath_back, dx, dy)

            for k in range(len(self.cps)-1):
                i = len(self.cps)-1 - k-1
                color = (theme.getcolor(settings['colors'], settings['levels'], self.grad[i]))
                colorlisth.append(color)
                theme.fillhpoly(devc, linkpoints[i], color)
            theme.fillhslopecontour(devc, spath_back, dx, dy, colorlisth)
            # draw the first polygon
            v_a = (int((self.cps[0][0]-int(self.min_d)) * kmsize),
                   int((self.cps[0][1]-self.min_h) * metersize))

            points = ((lef_mar + v_a[0], max_y - low_mar),
                    (lef_mar + v_a[0], max_y - low_mar - v_a[1]),
                    (lef_mar + v_a[0] + dx, max_y - low_mar - v_a[1] + dy),
                    (lef_mar + v_a[0] + dx, max_y - low_mar + dy))
            color = (theme.getcolor(settings['colors'], settings['levels'], self.grad[0]))
            theme.fillfirsthpoly(devc, points, color)
        vpolygons = []
        spath_pnts = []
        colorlistv = []
        for i in range(len(self.cps)-1):
            v_a = (int((self.cps[i][0] - int(self.min_d)) * kmsize),
                   int((self.cps[i][1] - self.min_h) * metersize))
            v_b = (int((self.cps[i+1][0] - int(self.min_d)) * kmsize),
                   int((self.cps[i+1][1] - self.min_h) * metersize))

            # points that delimitate the area to color
            points = [(lef_mar + v_a[0], max_y - low_mar),
                    (lef_mar + v_b[0], max_y - low_mar),
                    (lef_mar + v_b[0], max_y - low_mar - v_b[1]),
                    (lef_mar + v_a[0], max_y - low_mar - v_a[1])]
            points = [(p[0] + dx, p[1] + dy) for p in points]
            vpolygons.append(points)
            spath_pnts.append(polytoBezier(points[3],
               self.dercp[i] * (-metersize / kmsize),
               points[2],
               self.dercp[i+1] * (-metersize / kmsize)))
            color = (theme.getcolor(settings['colors'], settings['levels'], self.grad[i]))
            colorlistv.append(color)
        # add also the two lower points (those near km bar)
        spath_pnts = [vpolygons[0][0]] + spath_pnts + [vpolygons[-1][1]]
        theme.fillvslopecontour(devc, spath_pnts,
                                max_y - low_mar + self.min_h*metersize,
                                max_y - low_mar - (1000 - self.min_h)*metersize,
                                vpolygons, colorlistv)
        infotext_x = []
        for i in range(len(self.cps)-1):
            points = vpolygons[i]
            color = (theme.getcolor(settings['colors'], settings['levels'], self.grad[i]))
            theme.fillvpoly(devc, points, color)

            # draw gradient text
            font = settings['fgrad']
            if (points[1][0] - points[0][0] > devc.gettextwidth("%.1f%%" % self.grad[i])):
                theme.gradtext(devc, "%.1f%%" % self.grad[i],
                         points[0][0] + 3, points[0][1] - 20,
                         font)
            infotext_x.append(points[2][0] - dx - 4)
        infotext_x.append(infotext_x[len(self.cps)-2]+50)

        for i in range(len(self.cps)-1):
            # another cycle to prevent text to be hidden by polygons
            points = vpolygons[i]

            # draw info text
            font = settings['fdesc']
            infotext = "%.0f %s" % (self.cps[i + 1][1], self.cps[i + 1][2])
            diffx = infotext_x[i+1] - infotext_x[i]
            diffx = diffx - devc.gettextheight(infotext) - 2
            if diffx < 0:
                diffx = diffx/2
                infotext_x[i+1] -= diffx
                theme.desctext(devc, infotext,
                     infotext_x[i] + diffx, points[2][1] - dy - 10, font)
            else:
                theme.desctext(devc, infotext,
                     infotext_x[i], points[2][1] - dy - 10, font)


class SlopeList:
    """Wrapper for a list of slopes, according to MCV"""
    def __init__(self):
        """ Wraps a list of slopes, according to MCV"""
        self._lst = []
        self.message = glal.Message()

    def __len__(self):
        """"Gives how many slopes are in the list"""
        return len(self._lst)

    def new_slope(self):
        """ Add a new slope to the list. """
        self._lst.append(Slope())
        return (len(self._lst) - 1)

    def del_slope(self, slope_number):
        """ Remove a slope from the list """
        del self._lst[slope_number]
        # It doesen't send a SLOPE CHANGED message because
        # other slopes are not been modified.

    def get_slope_copy(self, slope_number):
        """ Get a copy of a slope in the list. """
        return self._lst[slope_number]

    def set_name(self, slope_number, name):
        """ Set the name of a slope in the list. """
        self._lst[slope_number].name = name
        # This updates the title in the tab.
        self.message.send("UPDATE_TAB", slope_number, 0)

    def set_country(self, slope_number, country):
        """ Set the country of a slope in the list."""
        self._lst[slope_number].country = country

    def set_author(self, slope_number, author):
        """ Set the author of a slope in the list."""
        self._lst[slope_number].author = author

    def set_email(self, slope_number, email):
        """ Set the email of the author of a slope."""
        self._lst[slope_number].email = email

    def set_comment(self, slope_number, comment):
        """ Ser a comment to a slope in the list. """
        self._lst[slope_number].comment = comment

    def set_url(self, slope_number, url):
        """ Add a URL referring to a slope in the list """
        self._lst[slope_number].url = url

    def get_name(self, slope_number):
        """ Get name """
        return self._lst[slope_number].name

    def get_state(self, slope_number):
        """ Get country """
        return self._lst[slope_number].country

    def get_author(self, slope_number):
        """ Get author """
        return self._lst[slope_number].author

    def get_email(self, slope_number):
        """ Get email """
        return self._lst[slope_number].email

    def get_comment(self, slope_number):
        """ Get comment """
        return self._lst[slope_number].comment

    def get_average_grad(self, slope_number):
        """ Get average gradient """
        return self._lst[slope_number].average_grad

    def get_max_grad(self, slope_number):
        """ Get max gradient """
        return self._lst[slope_number].max_grad

    def get_height_difference(self, slope_number):
        """ Get height difference"""
        return self._lst[slope_number].height_difference

    def get_height_gain(self, slopenumber):
        """ Get height gain"""
        return self._lst[slopenumber].height_gain

    def get_url(self, slope_number):
        """ Get URL """
        return self._lst[slope_number].url

    def get_coords(self, slope_number):
        """ Get coords """
        return self._lst[slope_number].coords

    def add_coord(self, slope_number, latitude, longitude):
        """ Add a coordinate to a slope in the list."""
        sel_lst = self._lst[slope_number]
        sel_lst.add_coord(latitude, longitude)

    def add_cp(self, slope_number, distance, altitude, name=""):
        """ Add a check point to a slope in the list."""
        sel_lst = self._lst[slope_number]
        orig_len = len(sel_lst)
        row_num = sel_lst.add_cp(distance, altitude, name)
        if len(sel_lst) == orig_len:
            # if the slope isn't grown then a cp has been modified.
            self.message.send("SLOPE_DEL", slope_number, row_num)
        self.message.send("SLOPE_ADD", slope_number, row_num)

    def remove_cp(self, slope_number, cp_num):
        """ Remove a check point from a slope in the list."""
        self._lst[slope_number].remove_cp(cp_num)
        self.message.send("SLOPE_DEL", slope_number, cp_num)

    def shift_slope(self, slope_number, shift_amount):
        sel_lst = self._lst[slope_number]
        sel_lst.shift(shift_amount)
        for row_num in range(len(sel_lst)):
            self.message.send("SLOPE_DEL", slope_number, row_num)
            self.message.send("SLOPE_ADD", slope_number, row_num)

# Below this line fuctions are part of the view according MCV pattern ###


def polytoBezier(p0, m1, p3, m2):
    """Covert from polynomial function to Beziér curve
    p0 is the start point (as tuple of dimension 2) of the Beziér curve
    p1 is the end point of the Beziér curve
    m1 is the value of the derivate in p0
    m2 is the value of the derivate in p3

    returns the four control points of a cubic Beziér curve (p1, p2, p3, p4)
    """
    (x0, y0) = p0
    (x3, y3) = p3
    h = x3 - x0

    x1 = x0 + h/3
    y1 = m1*(x1 - x0) + y0
    p1 = (x1, y1)

    x2 = x0 + 2*h/3
    y2 = m2*(x2 - x3) + y3
    p2 = (x2, y2)

    return (p0, p1, p2, p3)

# vim:sw=4:softtabstop=4:expandtab



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""altitude_downloader.py"""

# Copyright (C) 2008-2014 Federico Brega, Pierluigi Villani

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

import logging
import math
import threading
from urllib.request import urlopen
from urllib.error import URLError
import re

ALTITUDE_CACHE = {}
SERVICES = frozenset(("geonames.org", "earthtools.org", "usgs.net"))
_ACTIVE_SERVICE = "geonames.org"


def geonames_org(lat, lng):
    """ Use geonames.org STRM3 to download data"""
    url = 'http://api.geonames.org/srtm3?lat='+lat+'&lng='+lng+'&username=cyclograph'
    line = ''
    tries = 5
    # Sometimes geonames retuns an xml header, which is wrong, so try again.
    # the first character might be a minus sign, we keep it simple
    while not line.isdigit() and tries > 0:
        han = urlopen(url)
        line = han.readline().strip()
        tries -= 1
        han.close()
    alt = float(line)
    return alt


def earthtools_org(lat, lng):
    """ Use earthtools.org to download data"""
    # http://www.earthtools.org/height/<latitude>/<longitude>
    url = 'http://www.earthtools.org/height/'+lat+'/'+lng
    han = urlopen(url)
    alt = re.search(r"<meters>(-?\d+)<\/meters>", han.read()).group(1)
    alt = float(alt)
    han.close()
    return alt


def usgs_net(lat, lng):
    """ Use usgs.net to download data"""
    url = 'http://gisdata.usgs.net/XMLWebServices/' + \
          'TNM_Elevation_Service.asmx/getElevation?X_Value=' + \
          lng + '&Y_Value=' + lat + \
          '&Elevation_Units=METERS&Source_Layer=-1&Elevation_Only=TRUE'
    han = urlopen(url)
    match = re.search(r"<.+>(-?\d+[\.\d]*)<\/.+>", han.read()).group(1)
    alt = float(match)
    return alt

SERVER_DOWNLOADER = {"geonames.org": geonames_org,
                     "earthtools.org": earthtools_org,
                     "usgs.net": usgs_net
                     }


# WARNING! this module is not thread safe, maybe you can use a lock to use
# more threads.
def choose_service(newserv):
    """ Choose the service to use for retreiving altitude data"""
    global _ACTIVE_SERVICE
    global ALTITUDE_CACHE
    if newserv not in SERVICES:
        return -1

    if newserv != _ACTIVE_SERVICE:
        # Because different services may give different datas
        # we clear the cache if we change service.
        ALTITUDE_CACHE = {}
    _ACTIVE_SERVICE = newserv
    return 0


class ImporterThread(threading.Thread):
    """ Download altitude"""
    def __init__(self, outqueue, slopefile, num, start_dist, end_dist, seltype):
        threading.Thread.__init__(self)
        self.outq = outqueue
        self._want_abort = False
        self.slopefile = slopefile
        self.service = _ACTIVE_SERVICE
        self.status = 'OK'
        self.num_cps = num
        self.seltype = seltype
        self.start_dist = start_dist
        self.end_dist = end_dist

    def run(self):
        """ Task to execute while running"""
        if self.service not in SERVER_DOWNLOADER:
            self.status = "Error: service unknown"
            return
        getalt = SERVER_DOWNLOADER[self.service]

        self.slopefile.setcpToLoad(self.num_cps,
                                   self.start_dist,
                                   self.end_dist,
                                   self.seltype)
        cps = self.slopefile.cps
        usedcp = self.slopefile.cpToLoad

        for i in range(len(usedcp)):
            logging.debug("cpi:"+str(i)+"\tcp:"+str(cps[usedcp[i]]))
            (dist, alt, name, lat, lng) = cps[usedcp[i]]
            if (lat, lng) in ALTITUDE_CACHE:
                # altitude has already been downloaded
                alt = ALTITUDE_CACHE[(lat, lng)]
            else:
                # try to download the altitude
                try:
                    alt = getalt(lat, lng)
                except URLError:
                    self.status = 'Error: No network'
                    return
            ALTITUDE_CACHE[(lat, lng)] = alt
            progr = 1000 * i / (len(usedcp) - 1)
            self.outq.put((progr, (dist, alt, name)))
            if self._want_abort:
                self.status = 'Aborted'
                return

    def abort(self):
        """abort worker thread.
        Method for use by main thread to signal an abort.
        """
        self._want_abort = True


def point_conversion(lng, lat):
    """ return radians from coordinates"""
    return (math.radians(float(lng)), math.radians(float(lat)))


def distance(coords):
    """Calculates the distance from a list of coordinates."""
    (lng, lat) = coords[0]
    (lng_old, lat_old) = point_conversion(lng, lat)

    dist = 0.0
    cps = []
    for pnt in coords:
        if len(pnt) != 2:
            continue
        (lng, lat) = pnt
        (lng_new, lat_new) = point_conversion(lng, lat)
        dist += lambert_formulae(lat_old, lat_new, lng_old, lng_new)

        cps.append((dist, lng.strip('\n\t'), lat.strip('\n\t')))
        (lng_old, lat_old) = (lng_new, lat_new)
    return cps

R_EARTH = 6367.45  # km


def haversine_angle(lat_old, lat_new, lng_old, lng_new):
    """Haversine formula"""
    alpha = (math.sin((lat_old-lat_new)/2))**2 \
        + math.cos(lat_old) * math.cos(lat_new) * (math.sin((lng_old-lng_new)/2)**2)
    return 2 * math.atan2(math.sqrt(alpha), math.sqrt(1-alpha))


def haversine(lat_old, lat_new, lng_old, lng_new):
    return R_EARTH * haversine_angle(lat_old, lat_new, lng_old, lng_new)

R_EARTH_EQUATOR = 6378.137  # km WGS 84
r = 298.257223563  # inverse of eccentricity WGS 84


def lambert_formulae(lat_old, lat_new, lng_old, lng_new):
    red_lat_old = math.atan((r - 1) / r * math.tan(lat_old))
    red_lat_new = math.atan((r - 1) / r * math.tan(lat_new))

    sigma = haversine_angle(red_lat_old, red_lat_new, lng_old, lng_new)

    if sigma == 0.0:
        return 0.0

    P = (red_lat_old + red_lat_new) / 2
    Q = (red_lat_new - red_lat_old) / 2

    X = (sigma - math.sin(sigma)) * math.sin(P)**2 * math.cos(Q)**2 / (math.cos(sigma/2)**2)
    Y = (sigma + math.sin(sigma)) * math.cos(P)**2 * math.sin(Q)**2 / (math.sin(sigma/2)**2)

    distance = R_EARTH_EQUATOR * (sigma - (X + Y) / (2*r))
    return distance

# vim:sw=4:softtabstop=4:expandtab



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import os

def findandreplace(text):
    #PyQt5 is bad again :'( but we have a new trick: the header file
    text = text.replace(
        "QtCore.QCoreApplication.translate", "cg_translate")
    
    return text

ui_files = [ x  for x in os.listdir('.') if x.endswith('.ui')]
ui_files.sort()

commands = []
python_files = []
for uif in ui_files:
    python_file = 'ui_' + uif.rsplit('.')[0] +'.py'
    python_files += [python_file]
    commands += ['pyuic5 -o ' + python_file + ' ' + uif]

for command in commands:
    print(command)
    os.system(command)

print("Building ui_qt module: ../cyclograph/qt/ui_qt.py")
outfile = open('../cyclograph/qt/ui_qt.py','w')

with open('imports.py', 'r') as imports:
    outfile.write(imports.read())

for i in range(len(python_files)):
    filename = python_files[i]
    fid = open(filename, 'r')

    for line in fid:
        if line.startswith("from PyQt5 import") or line.startswith("import cyclograph"):
            continue

        res = findandreplace(line)
        outfile.write(res)
    fid.close()

outfile.close()
